# zksync
#   Copyright (C) 2017  Jonas Acres

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

module ZKSync
  class Archive
    attr_reader :fs_base_path, :keystore, :file_index, :root

    def initialize(passphrase, fs_base_path, root)
      @root = root
      @fs_base_path = fs_base_path
      @keystore = KeyStore.new(Key.new(passphrase, type: :passphrase))
      @file_index = CryptoFileIndex.new("@@index", self)
    end

    # add fs-paths to the archive. returns array of paths that were not added successfully.
    # archive paths are stored relative to basepath
    def add(fs_paths)
      [*fs_paths]
        .map { |fs_path| File.expand_path(fs_path) }
        .reject { |fs_path| add_file(fs_path) }
    end

    # list of all archive paths in this archive
    def list
      @file_index.list
    end

    def rm(*archive_paths)
      archive_paths.each { |archive_path| @file_index.rm(archive_path) }
    end

    def commit
      @file_index.write
    end

    def file_at_path(archive_path)
      @file_index.file_at_path(archive_path)
    end

    def inode_for_path(archive_path)
      @file_index.inode_for_path(archive_path)
    end

    ###

    def add_file(fs_path)
      archive_path = Pathname.new(fs_path).relative_path_from(Pathname.new(fs_base_path)).to_s
      @file_index.add(archive_path, fs_path)
    end
  end
end
