# zksync
#   Copyright (C) 2017  Jonas Acres

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

module ZKSync
  class CryptoInode
    attr_accessor :path, :mtime, :ctime, :atime, :mode, :uid, :gid, :size, :ftype, :sha256, :index

    def initialize(arg)
      if arg.is_a?(String)
        @path = arg
      else
        arg.each do |k,v|
          v = Time.parse(v) if [:atime, :mtime].include?(k.to_sym)
          self.send(:"#{k}=", v)
        end
      end
    end

    def inode_width
      CryptoFileIndex.inode_width
    end

    def slots_used
      (to_s.length.to_f/inode_width).ceil
    end

    def set_from_fs(fs_path, opts={})
      stat = File.stat(fs_path)

      @fs_path = fs_path
      @atime = stat.atime
      @mtime = stat.mtime
      @ctime = stat.ctime
      @ftype = stat.ftype
      @gid = stat.gid
      @mode = stat.mode
      @size = stat.ftype == "directory" ? 0 : stat.size
      @uid = stat.uid
      sha256 if opts[:hash]
    end

    def sha256
      return nil if @ftype == "directory"
      @sha256 ||= Digest::SHA256.file(@fs_path).hexdigest if @fs_path
      @sha256
    end

    def equals(other)
      [ :path, :mtime, :ctime, :atime, :mode, :uid, :gid, :size, :ftype, :sha256 ].each do |key|
        return false unless self.send(key) == other.send(key)
      end

      true
    end

    def to_h
      { path:path, mtime:mtime, ctime:ctime, atime:atime, mode:mode, uid:uid, gid:gid, size:size, ftype:ftype, sha256:sha256 }
    end

    def to_json
      to_h.to_json
    end

    # string is json right-space-padded to a multiple of inode_width, so we can reuse lines in @@index
    def to_s
      json = to_json
      json.length.to_s + "|" + json
    end
  end
end