# zksync
#   Copyright (C) 2017  Jonas Acres

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

module ZKSync
  class InodeTable
    attr_reader :inode_width

    def initialize(inode_width)
      @inode_width = inode_width
      @table = []
    end

    def add_inode(inode)
      num_slots_needed = inode.slots_used
      n = find_free(num_slots_needed)
      num_slots_needed.times { |i| @table[n+i] = i == 0 ? inode : :placeholder }
      inode.index = n
    end

    def delete_inode(index)
      return unless inode = @table[index]
      
      while index < @table.length && [:placeholder, inode].include?(@table[i]) do
        @table[i] = nil
        index += 1
      end
    end

    def find_free(size)
      (0..@table.length-size).to_a.each do |n|
        return n if free_at_n?(n)
      end

      @table.length
    end

    def free_at_n?(n)
      (0..n).to_a.each { |m| return false unless @table[m].nil? }
      true
    end

    def to_s
      # TODO: redo writing of InodeTable with a stream, so we don't have to have the whole serialized table in memory
      @table.map do |entry|
        case entry
        when nil
          " "*inode_width
        when :placeholder
          ""
        else
          text = entry.to_s
          text.to_s + " "*(inode_width - (text.length%inode_width)) unless text.length > 0 && text.length % inode_width == 0
        end
      end.join("")
    end
  end
end
