# zksync
#   Copyright (C) 2017  Jonas Acres

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.

#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

module ZKSync
  class Key
    def self.size
      32 # size in bytes of the key
    end

    def self.write_size
      return @write_size if @write_size

      k = Key.new("", type: :key)
      @write_size = k.encrypt(k.to_b).length
    end

    def initialize(passphrase, opts={})
      @raw_key = case opts[:type]
      when :literal
        passphrase
      when :passphrase
        kdf(passphrase)
      else
        Digest::SHA256.digest(passphrase)
      end

      @mode = opts[:mode] || :CBC
      @iv = Digest::SHA256.digest("iv" + @raw_key)
    end

    def kdf(passphrase)
      OpenSSL::PKCS5.pbkdf2_hmac_sha1(passphrase, "ZKSync", 2**16, self.class.size)
    end

    def child_key(string, opts={})
      Key.new(to_s + "::" + string, {type: :key}.merge(opts))
    end

    def write_size
      to_b.length
    end

    def encrypt(plaintext, padding=0)
      cipher = OpenSSL::Cipher::AES.new(8*self.class.size, @mode)
      cipher.encrypt
      cipher.key = to_b
      cipher.iv = @iv
      cipher.update(plaintext) + cipher.final
    end

    def decrypt(ciphertext)
      decipher = OpenSSL::Cipher::AES.new(8*self.class.size, @mode)
      decipher.decrypt
      decipher.key = @raw_key
      decipher.iv = @iv
      decipher.update(ciphertext) + decipher.final
    end

    def to_s
      @raw_key.unpack("H*").first
    end

    def to_b
      @raw_key
    end
  end
end
